import cachetools
import os
import functools
import hashlib

from diskcache import Cache

DEFAULT_CACHE_DIR = ".cache/"
DEFAULT_CACHE_EXPIRE = 60 * 60 * 24 * 7


class DiskCacheManager:

    def __init__(self, cache_name):
        parent_dir = os.getenv("CACHE_DIR") or DEFAULT_CACHE_DIR
        dir_str = os.path.join(parent_dir, cache_name)
        self.use_cache = os.getenv("USE_DISK_CACHE") is not None
        if self.use_cache:
            self.cache = Cache(dir_str)

    def memoize(self, expire=DEFAULT_CACHE_EXPIRE, not_expected_res=None):
        def decorator(func):
            def should_cache(res):
                if callable(not_expected_res):
                    return not not_expected_res(res)
                else:
                    return res is not not_expected_res

            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                if not self.use_cache:
                    return func(*args, **kwargs)

                cache_key = self._make_cache_key(func, *args, **kwargs)

                if cache_key in self.cache and should_cache(self.cache[cache_key]):
                    return self.cache[cache_key]
                else:
                    res = func(*args, **kwargs)
                    if should_cache(res):
                        self.cache.set(key=cache_key, value=res, expire=expire)
                    return res
            return wrapper
        return decorator

    def _make_cache_key(self, func, *args, **kwargs):
        """
        캐시 키 생성 함수

        :param func: 함수 객체
        :param args: 함수 인자
        :param kwargs: 함수 키워드 인자
        :return: 생성된 캐시 키 (문자열)
        """
        key_base = f"{func.__name__}:{args}:{kwargs}"
        return hashlib.md5(key_base.encode()).hexdigest()


def ttl_cache(maxsize=100, ttl=60):
    """
    TTL 캐시를 제공하는 데코레이터.

    :param maxsize: 캐시의 최대 크기.
    :param ttl: 캐시된 항목의 TTL(초 단위).
    """

    def decorator(func):
        cache = cachetools.TTLCache(maxsize=maxsize, ttl=ttl)

        @functools.wraps(func)
        @cachetools.cached(cache)
        def wrapped(*args, **kwargs):
            return func(*args, **kwargs)

        return wrapped

    return decorator



import json
import os

import pyqqq.config as c
from pyqqq.brokerage.ebest.domestic_stock import EBestDomesticStock
from pyqqq.brokerage.ebest.oauth import EBestAuth
from pyqqq.brokerage.ebest.simple import EBestSimpleDomesticStock
from pyqqq.brokerage.kis.domestic_stock import KISDomesticStock
from pyqqq.brokerage.kis.oauth import KISAuth
from pyqqq.brokerage.kis.simple import KISSimpleDomesticStock
from pyqqq.utils.api_client import send_request
from pyqqq.utils.logger import get_bare_logger as get_logger


logger = get_logger(__name__)


class NoConnection:
    """.env 파일 등 계정 정보가 없을 경우"""

    def __init__(self):
        pass


class KISConnection:
    """
    환경변수 파일(.env)에 한투 계정 정보가 있을 경우 브로커 연결을 생성하는 클래스입니다.

    Attributes:
        auth (KISAuth): 인증 객체
        broker_code (str): 브로커 코드
        broker (KISDomesticStock): 일반 거래용 브로커 객체
        broker_simple (KISSimpleDomesticStock): 간편 거래용 브로커 객체
        paper_auth (Optional[KISAuth]): 모의투자 인증 객체
        paper_broker_simple (Optional[KISSimpleDomesticStock]): 모의투자 간편 브로커 객체

    """

    logger = get_logger(__name__ + ".KISConnection")

    def __init__(self):
        app_key = os.getenv("KIS_APP_KEY")
        app_secret = os.getenv("KIS_APP_SECRET")
        account_no = os.getenv("KIS_CANO")
        account_product_code = os.getenv("KIS_ACNT_PRDT_CD")
        hts_id = os.getenv("KIS_HTS_ID")

        self.auth = KISAuth(app_key, app_secret)
        self.broker_code = "kis"
        self.broker = KISDomesticStock(self.auth)
        self.broker_simple = KISSimpleDomesticStock(self.auth, account_no, account_product_code, hts_id)
        self.paper_auth = None
        self.paper_broker_simple = None

        self.logger.info("Connected to KIS")
        if all(k in os.environ for k in ["PAPER_KIS_APP_KEY", "PAPER_KIS_APP_SECRET", "PAPER_KIS_CANO", "PAPER_KIS_ACNT_PRDT_CD"]):
            self.logger.info("Using paper broker")

            paper_app_key = os.getenv("PAPER_KIS_APP_KEY")
            paper_app_secret = os.getenv("PAPER_KIS_APP_SECRET")
            paper_account_no = os.getenv("PAPER_KIS_CANO")
            paper_account_product_code = os.getenv("PAPER_KIS_ACNT_PRDT_CD")

            self.paper_auth = KISAuth(paper_app_key, paper_app_secret, paper_trading=True)
            self.paper_broker_simple = KISSimpleDomesticStock(self.paper_auth, paper_account_no, paper_account_product_code)


class EBestConnection:
    """
    환경변수 파일(.env)에 LS(구 이베스트투자)증권 계정 정보가 있을 경우 브로커 연결을 생성하는 클래스입니다.

    Attributes:
        auth (EBestAuth): 인증 객체
        broker_code (str): 브로커 코드
        broker (EBestDomesticStock): 일반 거래용 브로커 객체
        broker_simple (EBestSimpleDomesticStock): 간편 거래용 브로커 객체
        account_no (str): 계좌 번호
        paper_auth (Optional[EBestAuth]): 모의투자 인증 객체
        paper_broker_simple (Optional[EBestSimpleDomesticStock]): 모의투자 간편 브로커 객체
    """

    logger = get_logger(__name__ + ".EBestConnection")

    def __init__(self):
        self.auth = EBestAuth(os.getenv("EBEST_APP_KEY"), os.getenv("EBEST_APP_SECRET"))
        self.broker_code = "ebest"
        self.broker = EBestDomesticStock(self.auth)
        self.broker_simple = EBestSimpleDomesticStock(self.auth)
        self.account_no = self.broker_simple.get_account().get("account_no")
        self.paper_auth = None
        self.paper_broker_simple = None

        self.logger.info("Connected to EBEST")
        if os.getenv("PAPER_TRADING") == "1":
            self.logger.info("Using paper broker")

            self.paper_auth = EBestAuth(os.getenv("EBEST_APP_KEY"), os.getenv("EBEST_APP_SECRET"), paper_trading=os.getenv("PAPER_TRADING") == "1")
            self.paper_broker_simple = EBestSimpleDomesticStock(self.paper_auth)


def get_base_class():
    if os.getenv("KIS_APP_KEY"):
        return KISConnection
    elif os.getenv("EBEST_APP_KEY"):
        return EBestConnection
    elif os.getenv("ACCOUNT_NICK"):
        return __set_account_info_env(os.getenv("ACCOUNT_NICK"))
    else:
        return NoConnection


def __lookup_account_info(nick: str):
    url = f"{c.PYQQQ_API_URL}/users/me/accounts/{nick}/access"
    r = send_request(
        "POST",
        url,
    )
    if r.status_code > 204:
        logger.error(f"Failed to get day data: {r.text}")
        r.raise_for_status()

    return json.loads(r.json())


def __set_account_info_env(nick: str):
    account = __lookup_account_info(nick)
    if account.get("KIS_APP_KEY"):
        os.environ["KIS_APP_KEY"] = account.get("KIS_APP_KEY") or os.getenv("KIS_APP_KEY")
        os.environ["KIS_APP_SECRET"] = account.get("KIS_APP_SECRET") or os.getenv("KIS_APP_SECRET")
        os.environ["KIS_CANO"] = account.get("KIS_CANO") or os.getenv("KIS_CANO")
        os.environ["KIS_ACNT_PRDT_CD"] = account.get("KIS_ACNT_PRDT_CD") or os.getenv("KIS_ACNT_PRDT_CD")
        os.environ["KIS_HTS_ID"] = account.get("KIS_HTS_ID") or os.getenv("KIS_HTS_ID")
        return KISConnection
    elif account.get("EBEST_APP_KEY"):
        os.environ["EBEST_APP_KEY"] = account.get("EBEST_APP_KEY") or os.getenv("EBEST_APP_KEY")
        os.environ["EBEST_APP_SECRET"] = account.get("EBEST_APP_SECRET") or os.getenv("EBEST_APP_SECRET")
        return EBestConnection
    else:
        return NoConnection


class PyQQQAutoConnectionSingleton(get_base_class()):
    """
    환경변수를 읽어서 자동으로 브로커 연결을 생성하는 싱글톤 클래스입니다.

    이 클래스는 환경변수에 따라 다음과 같은 클래스 중 하나의 인스턴스를 반환합니다:
    - NoConnection: 계정 정보가 없는 경우
    - KISConnection: 한투 계정 정보가 있는 경우
    - EBestConnection: LS(구 이베스트투자)증권 계정 정보가 있는 경우

    Example:
        >>> conn = PyQQQAutoConnectionSingleton()
        >>> conn.broker_simple.get_account()

    Returns:
        NoConnection | KISConnection | EBestConnection: 환경변수에 따라 결정된 브로커 연결 인스턴스
    """

    _instance = None
    _initialized = False

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self):
        if self._initialized:
            return
        self._initialized = True
        super().__init__()



import datetime as dtm
import os
from abc import ABC
from typing import Literal, Optional
from zoneinfo import ZoneInfo

from pyqqq.backtest.broker import BaseBroker, MockBroker, TradingBroker
from pyqqq.backtest.positionprovider import BasePositionProvider
from pyqqq.backtest.wallclock import WallClock
from pyqqq.brokerage.helper import EBestConnection, KISConnection
from pyqqq.brokerage.kis.oauth import KISAuth
from pyqqq.brokerage.kis.simple_overseas import KISSimpleOverseasStock

from pyqqq.utils.logger import get_handlers


class TradingEnvironment(ABC):
    """트레이딩 환경을 정의하는 추상 기본 클래스입니다.

    이 클래스는 트레이딩 시스템에서 사용되는 기본 환경을 정의하며,
    브로커와 시계 인스턴스를 제공하는 표준 인터페이스를 제공합니다.
    모든 구체적인 트레이딩 환경은 이 클래스를 상속받아야 합니다.
    """

    broker: BaseBroker
    """ 거래 실행을 담당하는 브로커 인스턴스 """

    clock: WallClock
    """ 시간 관리를 담당하는 시계 인스턴스 """

    def get_broker(self) -> BaseBroker:
        """현재 환경의 브로커 인스턴스를 반환합니다.

        Returns:
            BaseBroker: 거래 실행을 담당하는 브로커 인스턴스
        """
        return self.broker

    def get_clock(self) -> WallClock:
        """현재 환경의 시계 인스턴스를 반환합니다.

        Returns:
            WallClock: 시간 관리를 담당하는 시계 인스턴스
        """
        return self.clock


class BacktestEnvironment(TradingEnvironment):
    """과거 데이터를 사용한 백테스팅을 위한 거래 환경입니다.

    이 클래스는 지정된 기간 동안의 과거 데이터를 사용하여
    트레이딩 전략을 테스트할 수 있는 환경을 제공합니다.

    Example:

    .. highlight:: python
    .. code-block:: python

        env = BacktestEnvironment(
            start_time=datetime(2023, 1, 1, 9, 0),
            end_time=datetime(2023, 1, 31, 15, 30),
            time_unit="minutes",
            market="kr_stock"
        )
        await MyStrategy(env).run()

    """

    clock: WallClock
    """ 백테스팅용 가상 시계 인스턴스 """
    broker: MockBroker
    """ 백테스팅용 가상 브로커 인스턴스 """

    def __init__(
        self,
        start_time: dtm.datetime,
        end_time: dtm.datetime,
        time_unit: str = "minutes",
        position_provider: BasePositionProvider = None,
        market: str = "kr_stock",
        current_data_handling: Literal["include", "virtual", "exclude"] = "virtual",
        local_cache_path: str = "./data",
        market_nxt_on: bool = False,
    ):
        """BacktestEnvironment 클래스의 초기화 메서드입니다.

        Args:
            start_time (datetime): 백테스팅 시작 시간
            end_time (datetime): 백테스팅 종료 시간
            time_unit (str, optional): 시간 단위. Defaults to "minutes".
                                    "minutes" 또는 "days" 사용 가능.
            position_provider (BasePositionProvider, optional):
                초기 포지션 정보 제공자. Defaults to None.
            market (str, optional): 시장 유형. Defaults to "kr_stock".
                                          "kr_stock" 또는 "us_stock" 사용 가능.
            current_data_handling (Literal["include", "virtual", "exclude"], optional):
                현재 시점의 데이터(e.g. 분봉) 처리 방법. Defaults to "virtual".
                - "include": 현재(분)까지의 실제 데이터 반환
                - "virtual": 직전(1분전)까지의 데이터 + 현재(분)의 시가로 통일한 가상 데이터 반환 (기본값)
                - "exclude": 직전(1분전)까지의 데이터만 반환
            local_cache_path (str, optional): 백테스트 사용 데이터 로컬 캐시 경로. Defaults to "./data".
            market_nxt_on (bool, optional): NXT 시장 처리 여부. Defaults to False.

        Raises:
            AssertionError: start_time이 end_time보다 늦거나,
                          유효하지 않은 time_unit이 제공된 경우
        """
        super().__init__()

        assert start_time < end_time
        assert time_unit in ["minutes", "days"]

        tzinfo = None if market == "kr_stock" else ZoneInfo("America/New_York")

        self.clock = WallClock(live_mode=False, start_time=start_time, end_time=end_time, tzinfo=tzinfo)
        self.broker = MockBroker(self.clock, position_provider, market, time_unit, current_data_handling, local_cache_path=local_cache_path, market_nxt_on=market_nxt_on)

        # logger 에 백테스트 시간 정보 추간
        handlers = get_handlers()
        for handler in handlers:
            handler.update_format(clock=self.clock)


class KISDomesticEnvironment(TradingEnvironment):
    """한국투자증권 API를 사용한 국내주식 거래 환경입니다.

    실제 계좌 또는 모의투자 계좌를 사용한 거래가 가능하며,
    필요한 인증 정보는 환경변수에서 가져옵니다.

    필요한 환경변수:
        - KIS_APP_KEY: API 키
        - KIS_APP_SECRET: API 시크릿
        - KIS_CANO: 계좌번호
        - KIS_ACNT_PRDT_CD: 계좌상품코드

        모의투자 사용시 추가로 필요한 환경변수:
        - PAPER_KIS_APP_KEY: 모의투자 API 키
        - PAPER_KIS_APP_SECRET: 모의투자 API 시크릿
        - PAPER_KIS_CANO: 모의투자 계좌번호
        - PAPER_KIS_ACNT_PRDT_CD: 모의투자 계좌상품코드

    Example:
        ```python
        # 실제 계좌 사용
        env = KISDomesticEnvironment(paper_trading=False)

        # 모의투자 계좌 사용
        env = KISDomesticEnvironment(paper_trading=True)
        ```
    """

    def __init__(self, paper_trading: bool = False, market_nxt_on: bool = False, strategy_name: Optional[str] = None, classifier_type: Optional[str] = None):
        """KISDomesticEnvironment 클래스의 초기화 메서드입니다.

        Args:
            paper_trading (bool, optional): 모의투자 사용 여부. Defaults to False.
                                          True일 경우 모의투자 계좌 사용
            market_nxt_on (bool, optional): NXT 시장 처리 여부. Defaults to False.
            strategy_name (str, optional): 전략 이름. Defaults to None.
                position classifier 데이터가 저장되는 kvstore collection 이름에 사용됨.
            classifier_type (str, optional): "direct", "auto" 중 하나로 설정시 position classifier 사용하게 됨. Defaults to None.
                해당 값은 분류기가 기본 포지션을 어떤 방식으로 결정할지에 대한 설정.


        Raises:
            KeyError: 필요한 환경변수가 설정되지 않은 경우
        """
        conn = KISConnection()

        self.clock = WallClock(live_mode=True)
        self.broker = TradingBroker(
            data_api=conn.broker_simple,
            trading_api=conn.broker_simple if not paper_trading else conn.paper_broker_simple,
            clock=self.clock,
            market_nxt_on=market_nxt_on,
            strategy_name=strategy_name,
            classifier_type=classifier_type,
        )


class KISOverseasEnvironment(TradingEnvironment):
    """한국투자증권 API를 사용한 해외주식 거래 환경입니다.

    실제 계좌를 사용한 해외주식 거래가 가능하며,
    필요한 인증 정보는 환경변수에서 가져옵니다.
    현재 모의투자는 지원하지 않습니다.

    필요한 환경변수:
        - KIS_APP_KEY: API 키
        - KIS_APP_SECRET: API 시크릿
        - KIS_CANO: 계좌번호
        - KIS_ACNT_PRDT_CD: 계좌상품코드

    Example:
        ```python
        env = KISOverseasEnvironment()
        broker = env.get_broker()
        # 해외주식 거래 실행
        broker.create_order("AAPL", OrderSide.BUY, 10)
        ```
    """

    def __init__(self):
        """KISOverseasEnvironment 클래스의 초기화 메서드입니다.

        Raises:
            KeyError: 필요한 환경변수가 설정되지 않은 경우
        """
        self.clock = WallClock(live_mode=True)

        app_key = os.getenv("KIS_APP_KEY")
        app_secret = os.getenv("KIS_APP_SECRET")
        account_no = os.getenv("KIS_CANO")
        account_product_code = os.getenv("KIS_ACNT_PRDT_CD")

        auth = KISAuth(app_key, app_secret)
        simple_api = KISSimpleOverseasStock(auth, account_no, account_product_code)

        self.broker = TradingBroker(data_api=simple_api, trading_api=simple_api, clock=self.clock)


class EBestDomesticEnvironment(TradingEnvironment):
    """LS(구 이베스트투자)증권  API를 사용한 국내주식 거래 환경입니다.

    실제 계좌 또는 모의투자 계좌를 사용한 거래가 가능하며,
    필요한 인증 정보는 환경변수에서 가져옵니다.

    필요한 환경변수:
        - EBEST_APP_KEY: API 키
        - EBEST_APP_SECRET: API 시크릿

        모의투자 사용시 추가로 필요한 환경변수:
        - PAPER_TRADING=1

    Example:
        ```python
        # 실제 계좌 사용
        env = EBestDomesticEnvironment(paper_trading=False)

        # 모의투자 계좌 사용
        env = EBestDomesticEnvironment(paper_trading=True)
        ```
    """

    def __init__(self, paper_trading: bool = False):
        """EBestDomesticEnvironment 클래스의 초기화 메서드입니다.

        Args:
            paper_trading (bool, optional): 모의투자 사용 여부. Defaults to False.
                                          True일 경우 모의투자 계좌 사용

        Raises:
            KeyError: 필요한 환경변수가 설정되지 않은 경우
        """
        conn = EBestConnection()

        self.clock = WallClock(live_mode=True)
        self.broker = TradingBroker(
            data_api=conn.broker_simple,
            trading_api=conn.broker_simple if not paper_trading else conn.paper_broker_simple,
            clock=self.clock,
        )



import os
import datetime
from functools import wraps
from decimal import Decimal
import pandas as pd

from pyqqq.datatypes import *


def with_mock():
    """
    Decorator for mocking api call
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                if str(os.getenv('MOCKAPI')).lower() == 'true':
                    return mock_functions[func.__module__][func.__name__](*args, **kwargs)
                else:
                    return func(*args, **kwargs)
            except SkipMockGetException:
                return func(*args, **kwargs)
        return wrapper
    return decorator


def skip_mock_get(func):
    """
    Decorator for skipping mock api call
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        if str(os.getenv('MOCKAPI_SKIPGET')).lower() == 'true':
            raise SkipMockGetException()
        else:
            return func(*args, **kwargs)
    return wrapper


class SkipMockGetException(Exception):
    """
    Exception for skipping mock api call
    """


class MockEBestSimpleDomesticStock:
    @staticmethod
    @skip_mock_get
    def get_account(*args, **kwargs):
        return {'total_balance': 10000000, 'purchase_amount': 0, 'evaluated_amount': 0, 'pnl_amount': 10000000, 'pnl_rate': Decimal('0.000000')}

    @staticmethod
    @skip_mock_get
    def get_possible_quantity(*args, **kwargs):
        return {'investable_cash': 10000000, 'reusable_amount': 0, 'price': 70000, 'quantity': 142, 'amount': 9998000}

    @staticmethod
    @skip_mock_get
    def get_positions(*args, **kwargs):
        return []

    @staticmethod
    @skip_mock_get
    def get_historical_daily_data(*args, **kwargs):
        result = [
            {'date': datetime.date(2023, 3, 2), 'open': 60900, 'high': 61800, 'low': 60500, 'close': 60800, 'volume': 13049056},
            {'date': datetime.date(2023, 3, 3), 'open': 61000, 'high': 61200, 'low': 60500, 'close': 60500, 'volume': 10669655},
        ]
        df = pd.DataFrame(result)
        df['date'] = pd.to_datetime(df['date'])
        df.set_index('date', inplace=True)

        return df

    @staticmethod
    def create_order(*args, **kwargs):
        _arg_orders = ('self', 'asset_code', 'side', 'quantity', 'order_type', 'price')
        _args = {
            'asset_code': None,
            'side': None,
            'quantity': None,
            'order_type': OrderType.MARKET,
            'price': 0
        }
        for i, arg in enumerate(args):
            if i > 0:
                _args[_arg_orders[i]] = arg
        for k, v in kwargs.items():
            _args[k] = v

        def __get_ord_prc_ptn_code():
            order_type = _args['order_type']
            if order_type == OrderType.MARKET:
                return "03"
            elif order_type == OrderType.LIMIT:
                return "00"
            elif order_type == OrderType.LIMIT_IOC:
                return "00"
            elif order_type == OrderType.LIMIT_FOK:
                return "00"
            elif order_type == OrderType.MARKET_IOC:
                return "03"
            elif order_type == OrderType.MARKET_FOK:
                return "03"
            else:
                raise ValueError("지원하지 않는 주문 유형입니다.")

        def __get_ord_cndi_tp_code() -> int:
            order_type = _args['order_type']
            if order_type in [OrderType.MARKET, OrderType.LIMIT]:
                return 0
            elif order_type in [OrderType.MARKET_IOC, OrderType.LIMIT_IOC]:
                return 1
            elif order_type in [OrderType.MARKET_FOK, OrderType.LIMIT_FOK]:
                return 2
            else:
                raise ValueError("지원하지 않는 주문 유형입니다.")

        def __get_bns_tp_code() -> int:
            side = _args['side']
            if side == OrderSide.BUY:
                return 2
            elif side == OrderSide.SELL:
                return 1
            else:
                raise ValueError("지원하지 않는 주문 방향입니다.")

        __get_bns_tp_code()
        __get_ord_prc_ptn_code()
        __get_ord_cndi_tp_code()

        return 'MockEBestOrdNo001'

    @staticmethod
    def update_order(*args, **kwargs):
        _arg_orders = ('self', 'org_order_no', 'asset_code', 'order_type', 'price', 'quantity')
        _args = {
            'org_order_no': None,
            'asset_code': None,
            'order_type': None,
            'price': None,
            'quantity': 0
        }
        for i, arg in enumerate(args):
            if i > 0:
                _args[_arg_orders[i]] = arg
        for k, v in kwargs.items():
            _args[k] = v

        def __get_ord_prc_ptn_code():
            order_type = _args['order_type']
            if order_type == OrderType.MARKET:
                return "03"
            elif order_type == OrderType.LIMIT:
                return "00"
            elif order_type == OrderType.LIMIT_IOC:
                return "00"
            elif order_type == OrderType.LIMIT_FOK:
                return "00"
            elif order_type == OrderType.MARKET_IOC:
                return "03"
            elif order_type == OrderType.MARKET_FOK:
                return "03"
            else:
                raise ValueError("지원하지 않는 주문 유형입니다.")

        def __get_ord_cndi_tp_code() -> int:
            order_type = _args['order_type']
            if order_type in [OrderType.MARKET, OrderType.LIMIT]:
                return 0
            elif order_type in [OrderType.MARKET_IOC, OrderType.LIMIT_IOC]:
                return 1
            elif order_type in [OrderType.MARKET_FOK, OrderType.LIMIT_FOK]:
                return 2
            else:
                raise ValueError("지원하지 않는 주문 유형입니다.")

        __get_ord_prc_ptn_code()
        __get_ord_cndi_tp_code()

        return 'MockEBestOrdNo001'

    @staticmethod
    def cancel_order(*args, **kwargs):
        return 'MockEBestOrdNo001'


class MockKISSimpleDomesticStock:
    @staticmethod
    def create_order(*args, **kwargs):
        _arg_orders = ('self', 'asset_code', 'side', 'quantity', 'order_type', 'price')
        _args = {
            'asset_code': None,
            'side': None,
            'quantity': None,
            'order_type': OrderType.MARKET,
            'price': 0
        }
        for i, arg in enumerate(args):
            if i > 0:
                _args[_arg_orders[i]] = arg
        for k, v in kwargs.items():
            _args[k] = v

        def __get_order_type_code():
            order_type = _args['order_type']
            if order_type == OrderType.LIMIT:
                return "00"
            elif order_type == OrderType.MARKET:
                return "01"
            elif order_type == OrderType.LIMIT_CONDITIONAL:
                return "02"
            elif order_type == OrderType.BEST_PRICE:
                return "03"
            elif order_type == OrderType.PRIMARY_PRICE:
                return "04"
            elif order_type == OrderType.LIMIT_IOC:
                return "11"
            elif order_type == OrderType.LIMIT_FOK:
                return "12"
            elif order_type == OrderType.MARKET_IOC:
                return "13"
            elif order_type == OrderType.MARKET_FOK:
                return "14"
            elif order_type == OrderType.BEST_PRICE_IOC:
                return "15"
            elif order_type == OrderType.BEST_PRICE_FOK:
                return "16"
            else:
                raise ValueError("지원하지 않는 주문 유형입니다.")

        def __get_order_side_code():
            side = _args['side']
            if side == OrderSide.BUY:
                return "buy"
            elif side == OrderSide.SELL:
                return "sell"
            else:
                raise ValueError("지원하지 않는 주문 방향입니다.")

        __get_order_side_code()
        __get_order_type_code()

        return 'MockKISOrdNo001'

    @staticmethod
    def update_order(*args, **kwargs):
        _arg_orders = ('self', 'org_order_no', 'order_type', 'price', 'quantity')
        _args = {
            'org_order_no': None,
            'order_type': None,
            'price': None,
            'quantity': 0
        }
        for i, arg in enumerate(args):
            if i > 0:
                _args[_arg_orders[i]] = arg
        for k, v in kwargs.items():
            _args[k] = v

        def __get_order_type_code():
            order_type = _args['order_type']
            if order_type == OrderType.LIMIT:
                return "00"
            elif order_type == OrderType.MARKET:
                return "01"
            elif order_type == OrderType.LIMIT_CONDITIONAL:
                return "02"
            elif order_type == OrderType.BEST_PRICE:
                return "03"
            elif order_type == OrderType.PRIMARY_PRICE:
                return "04"
            elif order_type == OrderType.LIMIT_IOC:
                return "11"
            elif order_type == OrderType.LIMIT_FOK:
                return "12"
            elif order_type == OrderType.MARKET_IOC:
                return "13"
            elif order_type == OrderType.MARKET_FOK:
                return "14"
            elif order_type == OrderType.BEST_PRICE_IOC:
                return "15"
            elif order_type == OrderType.BEST_PRICE_FOK:
                return "16"
            else:
                raise ValueError("지원하지 않는 주문 유형입니다.")

        __get_order_type_code()

        return 'MockKISOrdNo001'

    @staticmethod
    def cancel_order(*args, **kwargs):
        return 'MockKISOrdNo001'


mock_functions = {
    'pyqqq.brokerage.ebest.simple': {
        'get_account': MockEBestSimpleDomesticStock.get_account,
        'get_possible_quantity': MockEBestSimpleDomesticStock.get_possible_quantity,
        'get_positions': MockEBestSimpleDomesticStock.get_positions,
        'get_historical_daily_data': MockEBestSimpleDomesticStock.get_historical_daily_data,
        'create_order': MockEBestSimpleDomesticStock.create_order,
        'update_order': MockEBestSimpleDomesticStock.update_order,
        'cancel_order': MockEBestSimpleDomesticStock.cancel_order,
    },
    'pyqqq.brokerage.kis.simple': {
        'create_order': MockKISSimpleDomesticStock.create_order,
        'update_order': MockKISSimpleDomesticStock.update_order,
        'cancel_order': MockKISSimpleDomesticStock.cancel_order,
    }
}
